/* multimon - CGI program to monitor several UPSes from one page

   Copyright (C) 1998  Russell Kroll <rkroll@exploits.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              
 */

#include <unistd.h>
#include <stdio.h>
#include <syslog.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "upsfetch.h"
#include "config.h"
#include "version.h"

void noresp()
{
	printf ("<TD BGCOLOR=\"#FF0000\">Unavailable</TD>\n");
}

void getinfo (char *monhost, char *desc)
{
	char	color[16], stattxt[128];
	float	tempf;
	char	model[64], status[64], battpct[64], utility[64], 
		loadpct[64], upstemp[64], *stat;

	printf ("<TR ALIGN=CENTER>\n");
	printf ("<TD BGCOLOR=\"#00FFFF\"><A HREF=\"upsstats.cgi?host=%s\">%s</A></TD>\n", monhost, desc);
	
	if (getupsvar (monhost, "model", model, sizeof(model)) < 0) {
		printf ("<TD COLSPAN=6 BGCOLOR=\"#FF0000\">Error: %s</TD></TR>\n",
		        upsstrerror(upserror));
		return;
	}
	
	printf ("<TD BGCOLOR=\"#00FFFF\">%s</TD>\n", model);

	if (getupsvar (monhost, "status", status, sizeof(status)) < 0) {
		noresp();
		return;
	}

	strcpy (stattxt, "");
	strcpy (color, "#FFFF00");	/* default: yellow */

	/* colors are set to red for bad situations and green for good ones */

	/* they are left alone for "in between" ones, and the default
	   (yellow) may be used in that case */

	stat = strtok (status, " ");
	while (stat != NULL) {	
		if (!strcasecmp(stat, "OFF")) {
			strcat (stattxt, "OFF<BR>");
			strcpy (color, "#FFFF00");
		}

		if (!strcasecmp(stat, "OL")) {
			strcat (stattxt, "ONLINE<BR>");
			strcpy (color, "#00FF00");
		}

		if (!strcasecmp(stat, "OB")) {
			strcat (stattxt, "ON BATTERY<BR>");
			strcpy (color, "#FF0000");
		} 

		if (!strcasecmp(stat, "LB")) {
			strcat (stattxt, "BATTERY LOW<BR>");
			strcpy (color, "#FF0000");
		}

		if (!strcasecmp(stat, "CAL")) {
			strcat (stattxt, "CALIBRATION<BR>");
		}
		stat = strtok (NULL, " ");
	} 

	printf ("<TD BGCOLOR=\"%s\">%s</TD>\n", color, stattxt);

	if (getupsvar (monhost, "battpct", battpct, sizeof(battpct)) > 0)
		printf ("<TD BGCOLOR=\"#00FF00\">%s %%</TD>\n", battpct);
	else
		printf ("<TD>\n");

	if (getupsvar (monhost, "utility", utility, sizeof(utility)) > 0)
		printf ("<TD BGCOLOR=\"#00FF00\">%s VAC</TD>\n", utility);
	else
		printf ("<TD>\n");

	if (getupsvar (monhost, "loadpct", loadpct, sizeof(loadpct)) > 0)
		printf ("<TD BGCOLOR=\"#00FF00\">%s %%</TD>\n", loadpct);
	else
		printf ("<TD>\n");

	if (getupsvar (monhost, "upstemp", upstemp, sizeof(upstemp)) > 0) {
		tempf = (strtod (upstemp, 0) * 1.8) + 32;
#ifdef USE_CELSIUS
		printf ("<TD BGCOLOR=\"#00FF00\">%s C</TD>\n", upstemp); 
#else
		printf ("<TD BGCOLOR=\"#00FF00\">%.1f F</TD>\n", tempf); 
#endif
		printf ("</TR>\n");
	}
	else
		printf ("<TD>\n");
}

int main() 
{
	FILE	*conf;
	time_t	tod;
	char	buf[256], fn[256], addr[256], *desc, timestr[256], *rest;
	int	restofs;

	printf ("Content-type: text/html\n");
	printf ("\n");

	printf ("<HTML>\n");
	printf ("<HEAD><TITLE>Multimon: UPS Status Page</TITLE></HEAD>\n");
	printf ("<BODY BGCOLOR=\"#FFFFFF\">\n");
	printf ("<TABLE BGCOLOR=\"#50A0A0\" ALIGN=CENTER>\n");
	printf ("<TR><TD>\n");
	printf ("<TABLE CELLPADDING=5>\n");
	printf ("<TR>\n");
	
	time (&tod);
	strftime (timestr, 100, "%a %b %d %X %Z %Y", localtime(&tod));
	printf ("<TH COLSPAN=7 BGCOLOR=\"#60B0B0\"><FONT SIZE=+2>Smart UPS Tools multimon %s</FONT><BR>%s</TH>\n", UPS_VERSION, timestr);
	printf ("</TR>\n"); 
	printf ("<TR BGCOLOR=\"#60B0B0\">\n");
	printf ("<TH COLSPAN=1>System</TH>\n");
	printf ("<TH COLSPAN=1>Model</TH>\n");
	printf ("<TH COLSPAN=1>Status</TH>\n");
	printf ("<TH COLSPAN=1>Batt Chg</TH>\n");
	printf ("<TH COLSPAN=1>Utility</TH>\n");
	printf ("<TH COLSPAN=1>UPS Load</TH>\n");
	printf ("<TH COLSPAN=1>Temperature</TH>\n");
	printf ("</TR>\n"); 

	/* ups status */

	snprintf (fn, sizeof(fn), "%s/etc/hosts.conf", BASEPATH);
	conf = fopen (fn, "r");

	while (fgets (buf, sizeof(buf), conf)) 
		if (strncmp("MONITOR", buf, 4) == 0) {
			sscanf (buf, "%*s %s %n", addr, &restofs);
			rest = buf + restofs + 1;
			desc = strtok (rest, "\"");
			getinfo (addr, desc);
		}

	fclose (conf);
	
	printf ("</TABLE>\n"); 
	printf ("</TD></TR>\n"); 
	printf ("</TABLE></BODY></HTML>\n");

	return (1);
}
